use crate::{
    Unit,
    measurement::Measurement,
};
use core::marker::PhantomData;
use core::ops::{
    Mul,
    Add,
};

#[macro_export]
macro_rules! prefix_group{
    ($(($nym: ident, $scale: expr)), *) => {
        $(
            pub struct $nym < T >(PhantomData < T >);
            impl < const TIME: i8,
            const LENGTH: i8,
            const MASS: i8,
            const CHARGE: i8,
            const TEMPERATURE: i8,
            const AMOUNT: i8,
            const INTENSITY: i8,
            T: Unit < Measurement < TIME,
            LENGTH,
            MASS,
            CHARGE,
            TEMPERATURE,
            AMOUNT,
            INTENSITY >>> Unit < Measurement < TIME,
            LENGTH,
            MASS,
            CHARGE,
            TEMPERATURE,
            AMOUNT,
            INTENSITY >> for $nym < T > {
                fn new(v: f64) -> Measurement < TIME,
                LENGTH,
                MASS,
                CHARGE,
                TEMPERATURE,
                AMOUNT,
                INTENSITY > {
                    Measurement::new(v.mul(Self::scale()).add(Self::offset()))
                }
                fn scale() -> f64 {
                    $scale * T:: scale()
                }
                fn offset() -> f64 {
                    T::offset()
                }
            }
        ) *
    };
}

prefix_group!{
    (Quetta, 1e30),
    (Ronna, 1e27),
    (Yotta, 1e24),
    (Zetta, 1e21),
    (Exa, 1e18),
    (Peta, 1e15),
    (Tetra, 1e12),
    (Giga, 1e9),
    (Mega, 1e6),
    (Kilo, 1e3),
    (Hecto, 1e2),
    (Deca, 1e1),
    (Deci, 1e-1),
    (Centi, 1e-2),
    (Milli, 1e-3),
    (Micro, 1e-6),
    (Nano, 1e-9),
    (Pico, 1e-12),
    (Femto, 1e-15),
    (Atto, 1e-18),
    (Zepto, 1e-21),
    (Yocto, 1e-24),
    (Ronto, 1e-27),
    (Quecto, 1e-30)
}
